/**
 * Copyright (c) 2008 by The Shoe Shine Boy.
 * 
 * This file is part of JMS SMTP Bridge.
 * 
 * JMS SMTP Bridge is free software: you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or (at your 
 * option) any later version.
 *
 * JMS SMTP Bridge is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with JMS SMTP Bridge. If not, see <http://www.gnu.org/licenses/>.
 */
package com.gt.jmsmtpbridge.smtp;

import java.rmi.RemoteException;
import java.util.Properties;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.NoSuchProviderException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import com.sun.mail.smtp.SMTPTransport;

/**
 * The SmtpSender SessionBean.
 * 
 * @author The Shoe Shine Boy
 *
 */
@SuppressWarnings("serial")
public class SmtpSenderBean implements SessionBean {
	// The logger
	private Logger m_logger = Logger.getLogger(getClass().getName());
	
	@Override
	public void ejbActivate() throws EJBException, RemoteException { }

	@Override
	public void ejbPassivate() throws EJBException, RemoteException { }

	@Override
	public void ejbRemove() throws EJBException, RemoteException { }

	@Override
	public void setSessionContext(SessionContext arg0) throws EJBException,
			RemoteException { }

	public void ejbCreate() { }
	
	/**
	 * The sendMail method. This method is called by the JmsMessageReceiver MDB for every message 
	 * that can be sent as a mail message.
	 * 
	 * This bean uses mail host related parameters (such as server name/IP, sender's email account, 
	 * etc. from the bean's initialization settings (defined in the ejb-jar.xml). After that if a 
	 * mail can be sent (based on whether important fields such as to, body, subject, etc. have been 
	 * passed to this method or not) it sends out the email. 
	 * 
	 * @param to				String[] of recipients that should be in the to field of the email
	 * @param cc				String[] of recipients that should be in the cc field of the email
	 * @param bcc				String[] of recipients that should be in the bcc field of the email
	 * @param subject			String representing the mail subject
	 * @param body				String representing the mail body
	 * @param sendAsHtml		boolean flag telling if the mail needs to be sent as html or plain text
	 * @throws RemoteException	Thrown in case of any fatal error
	 */
	public void sendMail(String[] to, String[] cc, String[] bcc, 
			String subject, String body, boolean sendAsHtml) throws RemoteException {
		m_logger.info("Inside sendMail().");
		String mailHost;
		final String acName;
		final String acPassword;
		boolean useAuthentication;
		boolean useTls;
		try {
			m_logger.info("Loading runtime parameters...");
			InitialContext ic = new InitialContext();
			mailHost = (String)(ic.lookup("java:/comp/env/mailHost"));
			m_logger.debug("Loaded mailHost: " + mailHost);
			acName = (String)(ic.lookup("java:/comp/env/acName"));
			m_logger.debug("Loaded account name: " + acName);
			acPassword = (String)(ic.lookup("java:/comp/env/acPassword"));
			useAuthentication = Boolean.parseBoolean((String)(ic.lookup("java:/comp/env/useAuthentication")));
			m_logger.debug("Loaded use Authentication flag: " + useAuthentication);
			useTls = Boolean.parseBoolean((String)(ic.lookup("java:/comp/env/useTls")));
			m_logger.debug("Loaded useTls: " + useTls);
		} catch (NamingException e1) {
			m_logger.error(e1.getMessage());
			throw new RemoteException(e1.getMessage());
		}

		// Connect to SMTP Server
		m_logger.info("Connecting to SMTP Server...");
		Properties props = new Properties();
		if (mailHost == null || mailHost.trim().length() == 0) {
			throw new RemoteException("mailHost is null or empty.");
		}
		props.setProperty("mail.host", mailHost);
		props.setProperty("mail.smtp.auth", String.valueOf(useAuthentication));
		props.setProperty("mail.smtp.starttls.enable", String.valueOf(useTls));
		if (acName == null || acName.trim().length() == 0) {
			throw new RemoteException("Mail Account Name is null or empty.");
		} else {
			// move on.
		}
		if (acPassword == null || acPassword.trim().length() == 0) {
			throw new RemoteException("Mail Account Password is null or empty.");
		} else {
			// move on
		}
		Session mailSession = Session.getInstance(props, new Authenticator() {
			protected PasswordAuthentication getPasswordAuthentication() {
				return new PasswordAuthentication(acName, acPassword);
			}
		});
		
		// Create Message
		Transport transport;
		MimeMessage message;
		Multipart multiPart;
		try {
			transport = (SMTPTransport)mailSession.getTransport("smtp");
			transport.connect(mailHost, acName, acPassword);
			m_logger.info("Connected to " + mailHost);

			message = new MimeMessage(mailSession);
			if (subject == null || subject.trim().length() == 0) {
				throw new RemoteException("Mail Subject is null or empty.");
			} else {
				message.setSubject(subject);
			}
			message.setFrom(new InternetAddress(acName));
			multiPart = new MimeMultipart();
			MimeBodyPart bodyPart = new MimeBodyPart();
			if (body == null || body.trim().length() == 0) {
				throw new RemoteException("Mail body is null or empty.");
			} else {
				// move on.
			}
			if (sendAsHtml) {
				bodyPart.setContent(body, "text/html");
			} else {
				bodyPart.setText(body);
			}
			multiPart.addBodyPart(bodyPart);
		} catch (AddressException e) {
			throw new RemoteException(e.getMessage());
		} catch (NoSuchProviderException e) {
			throw new RemoteException(e.getMessage());
		} catch (MessagingException e) {
			throw new RemoteException(e.getMessage());
		}
        try {
			message.setContent(multiPart);
			if (to == null || to.length == 0) {
				throw new RemoteException("List of recipients in TO field is null or empty.");
			} else {
				for (int i=0; i<to.length; i++) {
					message.addRecipient(Message.RecipientType.TO, new InternetAddress(to[i]));
				}
			}
			if (cc == null || cc.length == 0) {
				// skip CC
			} else {
				for (int i=0; i<cc.length; i++) {
					message.addRecipient(Message.RecipientType.CC, new InternetAddress(cc[i]));
				}
			}
			if (bcc == null || bcc.length == 0) {
				// skip BCC
			} else {
				for (int i=0; i<bcc.length; i++) {
					message.addRecipient(Message.RecipientType.BCC, new InternetAddress(bcc[i]));
				}
			}

			// Send email
			m_logger.info("Sending mail(s)...");
			Transport.send(message);
			m_logger.info("Mail(s) sent successfully.");
			
			// Close connection to the SMTP Server
			m_logger.info("Closing Connection to SMTP Server...");
			transport.close();
			m_logger.info("Mail(s) sent. Connection to SMTP Server closed.");
		} catch (AddressException e) {
			throw new RemoteException(e.getMessage());
		} catch (MessagingException e) {
			throw new RemoteException(e.getMessage());
		}
	}
}
